#!/usr/bin/env python
# -*- coding: utf8 -*-
from __future__ import absolute_import, division, generators, nested_scopes, print_function, unicode_literals, with_statement
from zenutils.sixutils import *

import time
import random
import string
import unittest
from zenutils import randomutils

class TestRandomUtils(unittest.TestCase):

    def test01(self):
        password = "".join(randomutils.choices(string.ascii_letters, k=random.randint(4, 16)))
        length = random.randint(1, 1024) * 2
        gen1 = randomutils.Random(password)
        gen2 = randomutils.Random(password)
        data1 = gen1.get_bytes(length)
        data2 = gen2.get_bytes(length)
        assert len(data1) == length
        assert data1 == data2
    
    def test02(self):
        password = "".join(randomutils.choices(string.ascii_letters, k=random.randint(4, 16)))
        length = random.randint(1, 1024) * 2
        gen1 = randomutils.Random(password)
        gen2 = randomutils.Random(password)
        data1 = gen1.get_bytes(length)
        data21 = gen2.get_bytes(length//2)
        data22 = gen2.get_bytes(length//2)
        assert data1 == data21 + data22

    def test03(self):
        gen1 = randomutils.Random(5)
        gen2 = randomutils.Random(5)
        length = 10
        data1 = [gen1.random() for _ in range(length)]
        data2 = [gen2.random() for _ in range(length)]
        assert data1 == data2

    def test04(self):
        gen = randomutils.Random(time.time())
        length = 10
        multi = 1000
        values = [0] * length
        for _ in range(length*multi):
            value = gen.randint(length)
            values[value] += 1
        for value in values:
            assert value
        a = min(values)
        b = max(values)
        assert b - a < b * 0.2

    def test05(self):
        gen = randomutils.Random()
        old = set()
        counter = 0
        while counter < 10000 * 100:
            value = gen.random()
            assert value not in old
            old.add(value)
            counter += 1

    def test06(self):
        gen = randomutils.Random()
        data = bytearray(gen.get_bytes(102400))
        for p in range(256):
            assert p in data

    def test07(self):
        gen = randomutils.Random()
        c1 = gen.choice(string.ascii_letters)
        c2 = gen.choice(string.ascii_letters)
        assert c1 in string.ascii_letters

    def test08(self):
        gen = randomutils.Random()
        length = 8
        pwd1 = "".join(gen.choices(string.ascii_letters, k=length))
        pwd2 = "".join(gen.choices(string.ascii_letters, k=length))
        assert len(pwd1) == length
        assert pwd1 != pwd2

    def test09(self):
        gen = randomutils.Random()
        length = gen.randint(1000)
        thelist = list(range(length))
        gen.shuffle(thelist)
        for x in range(length):
            assert x in thelist

    def test10(self):
        length = 10000 * 100
        gen = randomutils.Random()
        seeds = set()
        for x in range(length):
            value = gen.random()
            seeds.add(value)
        assert len(seeds) == length

    def test11(self):
        uuidgen = randomutils.UuidGenerator()
        uuids = set()
        for i in range(10000*10):
            uid = uuidgen.next()
            assert not uid in uuids
            uuids.add(uid)

    def test12(self):
        n = 100000
        uuidgen = randomutils.UuidGenerator()
        uuids = set(uuidgen.next(n))
        assert len(uuids) == n

    def test13(self):
        n1 = 0
        n2 = -1
        n3 = -100000
        uuidgen = randomutils.UuidGenerator()
        uuids1 = set(uuidgen.next(n1))
        uuids2 = set(uuidgen.next(n2))
        uuids3 = set(uuidgen.next(n3))
        assert len(uuids1) == 0
        assert len(uuids2) == 0
        assert len(uuids3) == 0

    def test14(self):
        thelist = ["a"]
        assert randomutils.choices(thelist, k=0) == []
        assert randomutils.choices(thelist, k=1) == ["a"]
        assert randomutils.choices(thelist, k=2) == ["a", "a"]
        assert randomutils.choices(thelist, k=3) == ["a", "a", "a"]


    def test15(self):
        password = "hello"
        value = 8141294968645153005348737041137229874925484556329662733687648891656190967473008484003711490685970494114845896430057820193510284132416327087093833302720579
        assert randomutils.Random.get_seed(password) == value

        password = "world"
        value = 917621815894390898271268045579761300172305340683275565896802420972662387591329011865249652589391928780576967173433776051555728720648442640225828065134318
        assert randomutils.Random.get_seed(password) == value

        password = ""
        value = 10868450558671247443152026947160338505683745266658651051718065983487878962987857602829315249215796444208488632888003673539585986066311769564391053988452926
        assert randomutils.Random.get_seed(password) == value

        password = "a"
        value = 1636897152625920295465748920275349232570945140773800183880278231231037334490840639395121881466231859643176795512242694658460339042012321358445155903773301
        assert randomutils.Random.get_seed(password) == value

    def test16(self):
        seed = randomutils.Random.get_seed(None)
        assert type(seed) == float

        seed = randomutils.Random.get_seed(12345678)
        assert seed == 12345678

        seed = randomutils.Random.get_seed(12345678.981)
        assert seed == 12345678.981
    
    def test17(self):
        data = []
        rnd = randomutils.Random(seed='hello world')
        for _ in range(1000):
            data.append(rnd.random())
        assert data == [0.2774731638676853, 0.19345671565105535, 0.7484835884455056, 0.9176353462359085, 0.5621010437006919, 0.8551780464103058, 0.0568540657487346, 0.6906870019613461, 0.7585108340166895, 0.5867050769630371, 0.9131828444689379, 0.0739050165906864, 0.8216765973298483, 0.4894820974354801, 0.7352293241684893, 0.6147333678813192, 0.9465476531812446, 0.5368928158519806, 0.7696947526675226, 0.007228990560665746, 0.5176018866571028, 0.9129232593044484, 0.9206688089252362, 0.704569972967464, 0.7449719131021632, 0.20673444852242098, 0.6746891206626545, 0.21859875793639003, 0.07552758082941036, 0.14821831040062772, 0.9208594122696797, 0.9577051939699617, 0.767194582180338, 0.0745636862224498, 0.6686456928010549, 0.9977689279344572, 0.43119528942356267, 0.10281672396223887, 0.43555925765617454, 0.7247777779747925, 0.08978233997049614, 0.05564606406787235, 0.10636144721978814, 0.6238163431530298, 0.34719048843461053, 0.9966077305966802, 0.7650536544717863, 0.29787126816486137, 0.9801279520213092, 0.9632128684133217, 0.918807320649059, 0.10907696573747563, 0.5721429507253595, 0.8343799031158649, 0.989624982732404, 0.9071285333537096, 0.9585503652386105, 0.9276843381161812, 0.9354862551764213, 0.3914051326404278, 0.6594967674505491, 0.41367133902275166, 0.2471942913901051, 0.3644466881150684, 0.7224929799627762, 0.37774376198535026, 0.40684260163898717, 0.2087872110476817, 0.49897579387497354, 0.9122640196054085, 0.44508049821641393, 0.39275089175140077, 0.08554075986176372, 0.5418098536053755, 0.20947068150547926, 0.1385722729609376, 0.4783386663032474, 0.30859860222391866, 0.49712185801964054, 0.1613799410937824, 0.24576132979144819, 0.6498283607673832, 0.24222160068707285, 0.9899326692784708, 0.5762799930258163, 0.7156738561908278, 0.3150050639888918, 0.742792022057706, 0.4446012210329475, 0.31591552523315025, 0.5041311167191153, 0.6015942589412875, 0.4970439963976058, 0.6691701495300596, 0.7483942159307277, 0.1572661171522346, 0.04783700319033277, 0.8173029661943058, 0.28131559703279685, 0.8197744688098527, 0.6459944989319881, 0.191038715396527, 0.5521748094482611, 0.07937530080345442, 0.422851881491525, 0.9272243274142153, 0.411029994775325, 0.734965204322881, 0.3427195202007908, 0.3949670729622793, 0.1475953451698425, 0.9072622714490123, 0.9885053723408411, 0.16757004446498067, 0.011972914438634002, 0.012904406127385352, 0.0548655430633719, 0.1454899754042246, 0.8389744799865934, 0.08875999394107148, 0.3931695373848748, 0.46463621017584583, 0.5100212054337205, 0.1926985106319497, 0.9253392615665155, 0.781072411809618, 0.02543024794909499, 0.6567283087889024, 0.0060626411922252215, 0.4965222087432069, 0.5595345046827199, 0.8103620518503786, 0.4618560879337774, 0.5988946479077207, 0.9888933634729188, 0.07525238973447193, 0.11739544531062214, 0.2792884796999604, 0.3714082421622295, 0.7295572247058115, 0.7734785495272126, 0.14051898056268536, 0.1903382265394883, 0.8041206166275061, 0.23985192587675788, 0.18166616591286394, 0.7810604244889234, 0.8817856835144318, 0.5234664128614643, 0.4104536110118243, 0.35999949246257595, 0.5216693779033049, 0.6814390573074043, 0.8735260598188888, 0.9394389061574439, 0.6185742252086264, 0.7735580889321374, 0.3321494441757282, 0.16692416155994136, 0.4437044346402232, 0.26469076978735245, 0.7364843149306507, 0.3550976939074841, 0.8539615204971522, 0.41264461505025607, 0.7236054569150561, 0.4657975564691035, 0.5480738898969006, 0.9082860935265553, 0.6263444865109296, 0.8411622462013264, 0.8409123753185715, 0.9267802403520788, 0.39312575271986433, 0.2400811099714678, 0.3992405739177655, 0.4058154950284037, 0.6380095650855879, 0.3710592845876177, 0.8682052210287537, 0.8722059917990675, 0.8060690866082008, 0.8891971469542185, 0.5770538864838058, 0.6120020320742486, 0.5913328443161894, 0.6724912390673222, 0.9628811294971308, 0.1532235534643225, 0.3480363051186037, 0.40621757286576055, 0.8900665997772743, 0.26328739611546637, 0.963741919587253, 0.6276534388360728, 0.024626472298375543, 0.8946135042345148, 0.025780290342187286, 0.15658318569818003, 0.5271628450469906, 0.2771590398022588, 0.3383278769385325, 0.9206250077694115, 0.24632402797879427, 0.9599890758820919, 0.08015700569095184, 0.48410147686458305, 0.2511311066625988, 0.9882409550820102, 0.40024986315552386, 0.9489949094014376, 0.9303747880060482, 0.38055905005368046, 0.24662729160960153, 0.31388559863192306, 0.41748704743659354, 0.48516945997752076, 0.03131524600691092, 0.5362269502766508, 0.06348707788556722, 0.16370302964020497, 0.020500350292419003, 0.4019634481040897, 0.9624350027145, 0.7242445456131724, 0.7577249678313533, 0.8330379962656295, 0.9570112701412886, 0.8690286605283915, 0.5880887590707992, 0.894245391661155, 0.2742196161126825, 0.8832898107665594, 0.5033370474780132, 0.14811786706164723, 0.6139428357676865, 0.5437621426094846, 0.6571989598056639, 0.12822926968505888, 0.29516729898023325, 0.49280802740868523, 0.7286743251622612, 0.28056884215919115, 0.9343274777178697, 0.9456103142516227, 0.028820231334112094, 0.8476726047151466, 0.6448678267480926, 0.5725473553902511, 0.8470494289013486, 0.4893784012280734, 0.30518435453206466, 0.3919648358456532, 0.6064598022045609, 0.27735446732740626, 0.25435086870222495, 0.4446567538003855, 0.24494048747418934, 0.6798994669244293, 0.5239323055812577, 0.7647915683321723, 0.43395545355932436, 0.14054389221633962, 0.31803531341415753, 0.5361766476481449, 0.48804039577580216, 0.19970339791311006, 0.762183866016442, 0.9857921025857058, 0.002827921463725196, 0.8227472439765584, 0.1441757983551489, 0.005584626066124996, 0.5517085856168729, 0.4044866927879721, 0.5997880381626735, 0.38410642854613286, 0.16585548187633414, 0.8730822109509431, 0.8493988536746857, 0.6663196664822553, 0.4240579018063267, 0.23623249113720524, 0.3331163016752908, 0.9877339429084632, 0.5334168163677112, 0.6414915357879203, 0.8658873159827846, 0.16494137828047803, 0.5237474083067272, 0.4925875547397914, 0.034749660722832885, 0.8252329772002099, 0.7024026091600368, 0.7010533465460232, 0.01264353769083859, 0.1773346154564037, 0.015125882713060642, 0.04208222155638629, 0.6341054976764509, 0.41237497626808267, 0.9043805966473594, 0.9482348046186397, 0.575120768680993, 0.14424509813918934, 0.6617770272697749, 0.25359143099210857, 0.1518043445586594, 0.2089576627784384, 0.26250390920022, 0.15470564933060343, 0.28775264711491927, 0.49175652019398175, 0.19187834268684545, 0.6623624875726126, 0.449563006660097, 0.9002819927642909, 0.9660651196865436, 0.09957088465061734, 0.8704195925546483, 0.37750619526357965, 0.831392510579158, 0.19173245838323086, 0.15784002407654688, 0.7629431159164809, 0.5070403172292521, 0.939753819541284, 0.572118182650876, 0.3942586962191114, 0.06062971491548637, 0.766247539011249, 0.34138606204629934, 0.707630420262496, 0.01838028428099095, 0.6520993502747194, 0.5125302157008677, 0.7348942225722761, 0.898241617877282, 0.15978567557188114, 0.17207408863512705, 0.08212059912039793, 0.11297622908050388, 0.9109461120756301, 0.021755979843776318, 0.08308142662802137, 0.2219805074608798, 0.18616784170037967, 0.34257161577972794, 0.26696525443584007, 0.09415496943921713, 0.14206194468870323, 0.014490207661224396, 0.14798137344234355, 0.77972073183947, 0.3764265801153661, 0.2978514261042152, 0.04711120137527609, 0.5744869133983581, 0.8418427005923776, 0.5899784796366263, 0.5762014516261623, 0.8176200175181584, 0.6859108493974656, 0.8179574664986173, 0.4267654700745228, 0.7082742674791798, 0.18877858209796627, 0.6731790586435553, 0.07175147355593726, 0.3447709009720066, 0.32393424556091444, 0.5603110853537834, 0.40402704313660626, 0.34411407464968485, 0.8220590149004092, 0.5762872461506048, 0.1248975846602893, 0.8647656338297679, 0.016592683268770447, 0.19346285008920328, 0.6091869523764806, 0.7887868577600036, 0.07253908773939571, 0.8555885252853991, 0.8590442367114205, 0.37230027427894363, 0.02057549142322064, 0.5814002290781322, 0.9374118746544616, 0.02415509356317845, 0.43025551606721757, 0.11098413970129872, 0.5528779544299313, 0.0026325684894638593, 0.12920068864374534, 0.6047321623200015, 0.7816111162758466, 0.5219932550495409, 0.14764362896625016, 0.5009914249742551, 0.40454516034928556, 0.8406503731461332, 0.9867955895596128, 0.536450400576401, 0.45730493517254905, 0.702725538890356, 0.1324157012630967, 0.7253592844058463, 0.15899270956255437, 0.5506657230601073, 0.05483833332968273, 0.6105059659925943, 0.4694313561590972, 0.6658928987050308, 0.7436747243352556, 0.4446632830686603, 0.2401900186425574, 0.35779707681821227, 0.4627686877586901, 0.13115078033577407, 0.9575575390145266, 0.19476732329679092, 0.4161785119225764, 0.4111340551638848, 0.9968447788587268, 0.47899483083362804, 0.3846905364331483, 0.11477481482844931, 0.9330548623374418, 0.4179795502091202, 0.9600723862818458, 0.09000105740413153, 0.37899386507576194, 0.7839051837131936, 0.3609937533329651, 0.558457309633953, 0.607121892992715, 0.2718696396332709, 0.2452852158121251, 0.9276842961928689, 0.003615553111501838, 0.05331598993177567, 0.49987861908787123, 0.08313912335801865, 0.5948418233377543, 0.1381750455641618, 0.00045094014674554593, 0.9522454227917054, 0.21401726641485594, 0.3541642560286175, 0.09966398624918839, 0.5232612109708166, 0.2561317715167538, 0.5633556048416796, 0.09019105238004715, 0.8800963706143554, 0.2895651114132839, 0.9286989692067902, 0.6098331601080805, 0.5395542420634103, 0.9961735176551393, 0.792122255705647, 0.3254368465682912, 0.5021663679251024, 0.1112948198068038, 0.874896707246305, 0.8411549615677131, 0.6996624475345601, 0.7071667492774338, 0.9529189234958021, 0.06003329174489959, 0.925254985895931, 0.8216147886222062, 0.04925939022535216, 0.7630753587051705, 0.9460681323497271, 0.8091379833639678, 0.8898347857153537, 0.9306360371541725, 0.34339225791034494, 0.28580101657602636, 0.8088126748437798, 0.7536109273255595, 0.5068543071589278, 0.06026749587650771, 0.5946631733572948, 0.07445452511697755, 0.6722463978657499, 0.3158728961627743, 0.2558711155011943, 0.7453982764315016, 0.9505250439505281, 0.08116856962550076, 0.7595589005552071, 0.22992575169139354, 0.3967204051988723, 0.5655579853124433, 0.5872817642780249, 0.8482260902323812, 0.16535300541195547, 0.638321185782544, 0.320536006500706, 0.39722957026657496, 0.34988942219384767, 0.34878069628514635, 0.11749003484096501, 0.8408763169119866, 0.3421173691933562, 0.705040768069298, 0.8073057942068512, 0.03755344481870718, 0.852085123365232, 0.9816631174431264, 0.9760560417500791, 0.8217938819461397, 0.46973207392336536, 0.8952967488406451, 0.900005299680079, 0.007732740548502031, 0.6092839712684106, 0.3051654439856424, 0.5542415683092118, 0.12744746337094648, 0.5926713791051854, 0.07025358508139294, 0.3211907428885873, 0.4167326720395249, 0.7521893966899486, 0.15233506474793967, 0.23968305467362058, 0.5590209181898924, 0.4547842899721401, 0.9602388190822365, 0.24369942842773265, 0.8992389270219391, 0.1679821084262961, 0.2311086223790501, 0.3394086551965079, 0.18268555550421772, 0.32497955251412247, 0.371495879321494, 0.045098144538143634, 0.3843290607222852, 0.07913146316084821, 0.8423222318844306, 0.5405690319884329, 0.9748792243164418, 0.2054241432422461, 0.974951065608998, 0.8101938248111399, 0.4634972469010299, 0.07859375095928167, 0.7731100691770263, 0.7618416165849011, 0.43000142274426645, 0.9766959547834214, 0.695663949730532, 0.6415587337968371, 0.1277785612727332, 0.3064783653942914, 0.45952036693342124, 0.572243170964936, 0.7501012955103694, 0.10815207393118227, 0.7823532494951847, 0.8638805673518077, 0.5960327790322386, 0.8400744783793698, 0.7627780744876475, 0.05058459134543615, 0.7996323747448636, 0.08631145306617716, 0.5850531305607843, 0.38019640022551404, 0.9865143809564574, 0.026813717829998373, 0.6751435728392996, 0.42959302799049504, 0.7544486597748398, 0.542761918136793, 0.056255596144362575, 0.08674374484593148, 0.34203035699235323, 0.4246128134853349, 0.8243509370578683, 0.2995134167818393, 0.5815675136739602, 0.38364878093097393, 0.5058864353785929, 0.8196936201824773, 0.773309669364349, 0.04437624687404622, 0.4431873982160727, 0.394544688451414, 0.9861866483873288, 0.5883480608170557, 0.5397269535731526, 0.04367590013910737, 0.6552087963608039, 0.36900369465190175, 0.3474979427981121, 0.7394367350545298, 0.7936335450154084, 0.2456548529948171, 0.9007128659158952, 0.4168684879672122, 0.2220868544464213, 0.12196618422362766, 0.3358569362432948, 0.11686985349047992, 0.7895493304089544, 0.6785578206236206, 0.5809177828082384, 0.5689224099869579, 0.5818231160789866, 0.9868081313770538, 0.534511607384749, 0.3273554083601951, 0.30616414741453857, 0.713952205401979, 0.1120087691379835, 0.2966786487674572, 0.6087894665239341, 0.2666404593876977, 0.32914443446551434, 0.29885566707991096, 0.04592794525165384, 0.18765881720688657, 0.7936303801415481, 0.05729128509605226, 0.7305278061395775, 0.9156244534503186, 0.8602079922830516, 0.33780543569280586, 0.47968636621491634, 0.7560984712579666, 0.13299398802128534, 0.24563705696507698, 0.5294934135683392, 0.7250190320744834, 0.7343993570953309, 0.7497740192031681, 0.47648685132008495, 0.4869712398532308, 0.8479554180166462, 0.36357708099962677, 0.9996875595999003, 0.12231915799287768, 0.44111851069839325, 0.5304674475293815, 0.6778655191480649, 0.7066677372859897, 0.4932005477681727, 0.032154522358532035, 0.4263994558140487, 0.622355632644831, 0.8927332805193117, 0.021852563062089936, 0.07218734615395597, 0.9667951734120875, 0.2767505479600904, 0.640861025102197, 0.4856851591954279, 0.24839248208231046, 0.42491500324849507, 0.007381048172936101, 0.2904650928139929, 0.21393982705689651, 0.6939836509293814, 0.05282002040841786, 0.11260311539898772, 0.15847417776941583, 0.058449215778559716, 0.38991458143786656, 0.05895307699922169, 0.018981269487248258, 0.26971068272682996, 0.6789590949065859, 0.6266850995723765, 0.3745075607042311, 0.009365517133334276, 0.8532538308622295, 0.39162765387981285, 0.18320700376141655, 0.9771487152132647, 0.23520268641202488, 0.6219856695542159, 0.19729283877595952, 0.8378292705673062, 0.02283214731752098, 0.2288777722187339, 0.22457057793120505, 0.23863019341464586, 0.9621051656062685, 0.5114516926135001, 0.4558932277590857, 0.9801172193339424, 0.24484541390300724, 0.4417913817043395, 0.9747935898028457, 0.7898858687664028, 0.7548828815549946, 0.928720590199651, 0.7450687945521757, 0.09980342668322385, 0.6712082889108412, 0.38983327438935544, 0.3013976279864964, 0.4972069388687049, 0.13801761870203386, 0.4173816769095996, 0.0848945952026991, 0.29670816564998076, 0.07901697618040503, 0.692343402191111, 0.04091615453957003, 0.5882339272828094, 0.861731520163344, 0.8480792463936753, 0.5750114379503221, 0.4557610143524411, 0.21123666184490866, 0.6217806172480334, 0.7289275284066935, 0.007976983320928352, 0.466888688665377, 0.42305967815021006, 0.6220446278776093, 0.47052048530309987, 0.43786364623501195, 0.8016008973252852, 0.7886308865758405, 0.29422810458453, 0.5699587290825366, 0.8498496741122509, 0.5880371379954944, 0.03212396103118886, 0.4758150245250538, 0.4062159459823034, 0.6041995210388108, 0.5698200623058332, 0.9809036383049871, 0.9986257524855714, 0.9535411554689492, 0.9657732682298527, 0.7246576233975026, 0.16807122479169465, 0.09877618487538808, 0.5978588904667145, 0.14911102433856235, 0.6380703571540692, 0.21399911969861482, 0.888848622972279, 0.20398081790014247, 0.751799563066171, 0.30784879409884486, 0.15520245819214093, 0.7477884145421382, 0.8702702126960549, 0.5790019218664667, 0.2711018168660837, 0.38127853503247144, 0.5592705869000321, 0.42624761237129377, 0.7820878958785272, 0.5829067410541683, 0.5849477672305757, 0.5732010900266404, 0.7025268204957033, 0.11972358498837046, 0.03098319869741405, 0.7930962125100336, 0.7547735709350805, 0.28132676696871906, 0.8517650732954627, 0.3346461976449514, 0.0183176867410549, 0.35617610223898183, 0.8365589470452565, 0.7960803662222842, 0.4415446545811797, 0.7068934758290726, 0.3829429091899295, 0.14023885187801424, 0.8726876236007917, 0.5191337374828741, 0.9826841170405122, 0.379120675143504, 0.7222766986827598, 0.40037699624188744, 0.16718022643473998, 0.9531762365728841, 0.7838291051544072, 0.13358454501606423, 0.16650233261361452, 0.6698497398510103, 0.07631105100174322, 0.5976932607893526, 0.9863275126358332, 0.6319436336007962, 0.3269157492669663, 0.12674850441105429, 0.7062353476402974, 0.8727857242549593, 0.1480650324376917, 0.8396931221356784, 0.5853604357944241, 0.6640476790877077, 0.9191184528065454, 0.13591429686833997, 0.8600495827321137, 0.10850798023273152, 0.07027085096316994, 0.9950860058362381, 0.5336317578695234, 0.6816364503077902, 0.13074540413715188, 0.8682741102802537, 0.5538015859791836, 0.5592840657864737, 0.29304417442026015, 0.641696495880943, 0.2931880547051451, 0.5606611815520884, 0.7682610229729031, 0.7903430700340631, 0.5582217720287204, 0.14830234342555842, 0.7298840577054236, 0.06630857066057039, 0.948529515484304, 0.6229794262667467, 0.5133436868429553, 0.35427996142218715, 0.5905528721575029, 0.7445849556165026, 0.3878798331969912, 0.5781437939864874, 0.010307156713879686, 0.6312792395818555, 0.3533972493697122, 0.7099729556167219, 0.3805323385987077, 0.4969864819856153, 0.32577965380570434, 0.5645855507526638, 0.19873050976805742, 0.9032574636545834, 0.41079318159224754, 0.39349058777986434, 0.5635561950996602, 0.7046332563168791, 0.4583835697807199, 0.6628954279698566, 0.00686586070041701, 0.5824990581648275, 0.789153287593829, 0.6498785301922625, 0.3104929594091192, 0.19455815107747468, 0.9183453442966142, 0.155928572330726, 0.9304703025426933, 0.7881032190374463, 0.4201108984192229, 0.7935433086844155, 0.19089861522263765, 0.0030300644117033123, 0.6790393661390629, 0.2121458939734007, 0.6984936555162007, 0.0965715843710888, 0.6612807758711723, 0.12895903013361823, 0.042220910795492866, 0.7182940388459645, 0.0658909479126865, 0.2854935871248702, 0.5044567269356977, 0.37166730090622574, 0.04655589071546116, 0.7794313658465126, 0.9675179979080829, 0.97534947968088, 0.7825305191295034, 0.7352334179678349, 0.6058603393916773, 0.774806127484653, 0.9070685038203415, 0.8414043066476037, 0.08593274260022254, 0.41674253703084696, 0.23901165694667512, 0.7074659243880724, 0.951526262179603, 0.8542636074746714, 0.27588092585926194, 0.4360089569995088, 0.8958902301615852, 0.9062878599083302, 0.5141002031522611, 0.022312041460024443, 0.8228171087287188, 0.883031943666065, 0.16145520131247862, 0.8845363141693438, 0.5940773237157183, 0.48572938671160015, 0.5838934045567096, 0.33555870030024654, 0.034801730001041385, 0.4703688851455158, 0.06733305272209175, 0.10945501735841769, 0.7860407848943965, 0.6583812407262282, 0.6063867162814385, 0.24158169567003923, 0.11852155573961651, 0.419442602544864, 0.6644900081038769, 0.16308834467019967, 0.25436725985533115, 0.37569024249078414, 0.6357117067196019, 0.7611028517434942, 0.8839283379240191, 0.42389722548817177, 0.23380999992152024, 0.8469489538665673, 0.35708059308351675, 0.6398615091908489, 0.6619977959202004, 0.7279941258224242, 0.8759020821999831, 0.023298906194742317, 0.04654573093588438, 0.1079217169771463, 0.9684041143984052, 0.6560819843158806, 0.9675263597796268, 0.9702129789794011, 0.7918524800402575, 0.1618562087152924, 0.6798397677696959, 0.36137746951458827, 0.028161751941034138, 0.9848862262764693, 0.4515138342989578, 0.7036508959977046, 0.24214777669643112, 0.44477947724295147, 0.7741727135117671, 0.9460006952686735, 0.18103343470739974, 0.7186136458256913, 0.48269140184285686, 0.5646163156727588, 0.801043085973423, 0.42423319260480225, 0.09557899080289849, 0.09101415677429814, 0.2813701046522277, 0.9031388500074873, 0.8593851520583068, 0.8335987297821156, 0.3275932816954073, 0.9704935773373474, 0.6124491443116055, 0.04753303074887267, 0.5616729155828128, 0.9709569316368366, 0.560121505597646, 0.21780996652567583, 0.7874736827110664, 0.5273215755947389, 0.7768012018007727, 0.6128681501117902, 0.25409400750347433, 0.33897792929963855, 0.25462869020926615, 0.8126865431564586, 0.6197969167981504, 0.875388176979923, 0.5305528917181848, 0.06622849724782481, 0.13238831815201782, 0.6248608817635629, 0.5329268167312033, 0.9561801697966804, 0.1735064776471922, 0.9102520209389146, 0.5884371162397268, 0.9793291423400319, 0.6576174091594698, 0.5069425943935034, 0.95160108809616, 0.6505301620254292, 0.2583501692544857, 0.3140478610357341]

    def test18(self):
        data = []
        rnd = randomutils.Random(seed=1234567890)
        for _ in range(1000):
            data.append(rnd.random())
        assert data == [0.8132623972925475, 0.5573442073576711, 0.3013447551413788, 0.6395314541520672, 0.6387643332257236, 0.19653836738512703, 0.17616854286293973, 0.18489252414168464, 0.6456399084110013, 0.858571859559127, 0.40874543334441943, 0.14539681429037268, 0.4707766082783995, 0.9563069784303617, 0.3958664387949666, 0.08617158694723048, 0.8970893223722674, 0.08548734792969429, 0.5236667133463581, 0.34422918401757685, 0.723570974653788, 0.8339797862704964, 0.8403245949338455, 0.3382268350245471, 0.7514790825960871, 0.7664635905976641, 0.6351439650071791, 0.27170689378779317, 0.9495506483625273, 0.5717700014646436, 0.17549111929474606, 0.6939285333266917, 0.41120460125757413, 0.9223886356484846, 0.17072066751726947, 0.09630565741699829, 0.11893287886982051, 0.7493652540286286, 0.30703061366961953, 0.23808296186606343, 0.9826518690452326, 0.292904997627175, 0.12691400186130772, 0.8494088410118541, 0.23914132083654885, 0.5293313342625614, 0.22557396816717806, 0.47201568677927985, 0.871309274835937, 0.420313198606813, 0.15792646659687032, 0.6077756818266175, 0.18759761388604052, 0.616693048499562, 0.9497789419417534, 0.1642923295815857, 0.3077727025255595, 0.4297903182752871, 0.7268480846102785, 0.5045137975872722, 0.15013088568751576, 0.7256168962226378, 0.04039587286767613, 0.3998864751883851, 0.1438258200244667, 0.49141999343417897, 0.9061362443147316, 0.4365292552662373, 0.36986474137280123, 0.9549864804651779, 0.595281204305433, 0.45241717959279065, 0.5454886150889122, 0.2797896140002083, 0.47767918797069203, 0.6857959733216743, 0.39674744712924864, 0.3181058100488405, 0.7673567356214432, 0.5890010878119364, 0.07721230507542401, 0.31521649941484614, 0.23599040167058558, 0.5985429930405494, 0.3976229752196557, 0.32168381541045266, 0.361511783642079, 0.6649538271578668, 0.15781417854208857, 0.3583438165223832, 0.8694597940892376, 0.058498491786491996, 0.384463652401529, 0.29460624142757835, 0.334361453161866, 0.08087356760415393, 0.34071800014554765, 0.39914813015016093, 0.20108405193835785, 0.8533689626341695, 0.47924610442413634, 0.6561093106154887, 0.7026525521146902, 0.5999294286270114, 0.16175512951651133, 0.8947859824353459, 0.5845873822523402, 0.12893228230739, 0.13279325130041247, 0.5038070684957501, 0.7183713060721519, 0.7824810353074562, 0.9812737362431081, 0.022013191852263223, 0.3443865122333138, 0.05353063186652998, 0.9693297114111211, 0.5480507359065351, 0.3634559383444505, 0.7410388072981292, 0.6816820402332103, 0.7029112761592647, 0.8050130221934315, 0.9901492267448764, 0.4506176340111427, 0.28846732283155946, 0.5901893066363159, 0.2885396453378531, 0.4599369582316468, 0.24327006658612135, 0.029682726790866212, 0.7937965323448626, 0.11180905857250614, 0.9034877198061306, 0.8951156525431587, 0.41582062052746066, 0.9867944025385116, 0.8692496964203136, 0.49337119601124035, 0.912220883717182, 0.1479771512841346, 0.7003580084332615, 0.9772480340715208, 0.10067352526569952, 0.778355246846782, 0.024288979996309697, 0.5237343839305246, 0.5579919382374425, 0.5440377503303504, 0.7049827979778656, 0.9479435786499832, 0.49983045858188196, 0.04702427590645662, 0.5985296101055791, 0.43839090163920613, 0.9014295932663372, 0.874565786282887, 0.4708218924231809, 0.8554032246290638, 0.18253090206300637, 0.48191982674426814, 0.2008499748746631, 0.6276387716841135, 0.8693051105613451, 0.19347403705287075, 0.1429524069114211, 0.4738822338115055, 0.2507021578040782, 0.31681554003841966, 0.46791902609952224, 0.11019586821040796, 0.8553452736528433, 0.8671938149706003, 0.42277895179076613, 0.8916411597706133, 0.5729970123340888, 0.8485373937242062, 0.7376308041097513, 0.3916937450556782, 0.814950714749179, 0.7767617169058838, 0.21849069139299362, 0.5825062231279848, 0.18019813364335183, 0.6478816459089644, 0.9070421808334534, 0.04980857843713154, 0.6839650299437865, 0.5489843680522662, 0.11316387013507274, 0.123779679510511, 0.5345662434140201, 0.5244078739315733, 0.06008817770539241, 0.9103481187538626, 0.2207276307001939, 0.9445940908241837, 0.3173753139244006, 0.7050428503945482, 0.49048889233542325, 0.8816681132084092, 0.9013264877845992, 0.38765881265306645, 0.3359557257890012, 0.7021296706462107, 0.3603819647765351, 0.9412798236600225, 0.44230724707358643, 0.1494942228136776, 0.6223388468190734, 0.90577947963844, 0.5710950300957798, 0.041963466210860514, 0.3341559815146452, 0.2399207602739014, 0.4750562973723715, 0.8872465812153649, 0.5376149145199065, 0.06994231239646712, 0.13047502935622166, 0.5725802217615091, 0.6819130354703834, 0.2049441693600919, 0.0437594858553376, 0.1276852738915889, 0.8943441593529008, 0.9480258060966983, 0.8025671703129974, 0.7687970666951186, 0.8052940818964796, 0.1555221501279631, 0.5822772863366998, 0.43863302235202106, 0.23690091455148585, 0.14151639615942108, 0.348942364404862, 0.3160417309694565, 0.7061475936459857, 0.22272974755065914, 0.9140154299447062, 0.2798572651027426, 0.9644670493406374, 0.31711710169020546, 0.041519927544042275, 0.1242892243360904, 0.9280272096085904, 0.07975179754914327, 0.3038245385485773, 0.5167330662584043, 0.9895460732891431, 0.24875739400720026, 0.1262243468252505, 0.31467646511094816, 0.7654103110361358, 0.18414529686262654, 0.8930993968510222, 0.07087022887859945, 0.1092248666442447, 0.9454946841537561, 0.13228715190513277, 0.3776006805922984, 0.36620105377986606, 0.12033207288345116, 0.6625264820303345, 0.164310722027617, 0.46339835030727367, 0.045364071706083864, 0.033449881372376436, 0.6119345619889346, 0.4845466776405506, 0.33468397958126195, 0.143316994153866, 0.0646481861066778, 0.3348267026068499, 0.2548380478819867, 0.9054616824953067, 0.7867460795499144, 0.8912708659364109, 0.7407484261473841, 0.41512470767168685, 0.9882339372096147, 0.45008145263263044, 0.5431694228007645, 0.8659903060697214, 0.03643302480934269, 0.878839820333999, 0.8981034151601779, 0.9909362366073433, 0.2302950623629272, 0.34614811738933593, 0.053497795592840965, 0.16186695815601787, 0.7629837117271082, 0.5700080427943133, 0.669533272686821, 0.3365160164431632, 0.9561926940061689, 0.38010145518900773, 0.9244233900391549, 0.7220581741536828, 0.39225432360819373, 0.8136456452817171, 0.263461349686508, 0.15943928983550806, 0.40261228749152317, 0.6713910157547701, 0.9887752326856041, 0.6106748136451228, 0.33350435985094906, 0.3695736977387044, 0.9174317895222891, 0.8274347332938832, 0.4545529934638309, 0.03668377272788333, 0.6877520269806656, 0.4972201808475279, 0.7193220284323896, 0.49725634690846465, 0.05467496450065713, 0.2656122697383416, 0.008453450977832939, 0.7765440933143465, 0.03904348718456287, 0.39579224679398256, 0.7973097502831733, 0.024217527935369347, 0.2582329418368826, 0.8925037100456039, 0.8892922232058513, 0.1534829645482065, 0.5846498125759196, 0.9354804007904763, 0.3578328995451855, 0.37507038427059025, 0.919954015194889, 0.6831294879671798, 0.19313553662641234, 0.42612509752806077, 0.4043173216077795, 0.34383679835196557, 0.6713320904352975, 0.19405949466950795, 0.997732190640283, 0.8850710149330636, 0.0839201602511972, 0.13352585903154413, 0.8692211104595329, 0.822617580567321, 0.5622871161612958, 0.8854773601281941, 0.9830091612143818, 0.5299969331047413, 0.7547141617741997, 0.9096645597982118, 0.395080172802954, 0.713219342251535, 0.2184078619516884, 0.710177116853763, 0.9474101825048566, 0.3923841449471957, 0.24821137184228667, 0.9793674955182017, 0.9976927553329937, 0.24926266974235559, 0.007559203270647264, 0.26441372548432984, 0.4182790884571011, 0.26935051313357733, 0.7741558898920131, 0.0889003707839611, 0.08024775283141178, 0.5806733364257758, 0.05095963691732329, 0.12472821803516515, 0.8831123645219037, 0.4522930303239934, 0.11615594654395715, 0.5299825677475837, 0.28237570266950296, 0.4585999892411178, 0.6322791026372885, 0.502489506203284, 0.6684201497527262, 0.7165093195698009, 0.42787878407402985, 0.7252397202974747, 0.05222675269372775, 0.7576834603544012, 0.5846248102742938, 0.5482007413451496, 0.5075112324401841, 0.9713021164931674, 0.7427585699139395, 0.1396605411030255, 0.36107316498558245, 0.9646990641718275, 0.8917614395078205, 0.9857436734962148, 0.8657843331732906, 0.927000710685986, 0.9212160506088072, 0.7443168588466312, 0.4792433095384413, 0.19896075712047198, 0.9942651968365094, 0.19879655254728013, 0.3710644731625042, 0.46929093485718826, 0.780892089177217, 0.8241178429888957, 0.8727501963397096, 0.4321685077965043, 0.8410686569109358, 0.6886542439538612, 0.9803838727325687, 0.49333985335695346, 0.8532785515430934, 0.1686111298133204, 0.020072124500267914, 0.7500015497796066, 0.10971057778231087, 0.4284600922080116, 0.7867993283966334, 0.4826069941318628, 0.6787654286029399, 0.07010738936330631, 0.3306847389696711, 0.20167440006338666, 0.4877561297041656, 0.12808387953127937, 0.5144808161050783, 0.08238285304401761, 0.7496884122697198, 0.8314275461507628, 0.08560490488623138, 0.48054840395169673, 0.16743452002401799, 0.0783434452649513, 0.762113342814872, 0.4900436059445905, 0.05150993311122707, 0.2494854699221401, 0.8032887606160756, 0.24780169598309465, 0.11460445964390961, 0.3740414787689342, 0.1817088385750803, 0.7942696724471685, 0.008312529180669716, 0.9995059685786458, 0.627739842878479, 0.6578380223163233, 0.3450337647965971, 0.4778119066661411, 0.7400832594087757, 0.6307710724651578, 0.8355481277983523, 0.3353626561036505, 0.2211846757920147, 0.7709724071325232, 0.4969978507827574, 0.647094099619391, 0.2949301549803818, 0.9779837341371997, 0.4363320357460715, 0.7295018780591862, 0.7726255326026632, 0.7262285606755366, 0.6939803573898138, 0.6230167358603785, 0.8025917610881319, 0.6852107383731207, 0.7178091616524417, 0.9626997025353188, 0.5718052055505294, 0.206778708201017, 0.06855123510724423, 0.8127804335972216, 0.133281135754526, 0.100707428868107, 0.4918658773269411, 0.06274836725834304, 0.6862034287362108, 0.6167552056226991, 0.35020826521376586, 0.3284518375416192, 0.9103448245933318, 0.480884004377216, 0.9441731150940785, 0.5454418358314892, 0.8866140079819067, 0.3489358276958855, 0.6057278315727991, 0.008981535249300138, 0.7211190838427755, 0.4378532586208, 0.3746446776971958, 0.08752615857464484, 0.32027331084393074, 0.20778033569388013, 0.18276833059508268, 0.06070320743380656, 0.8107430689436745, 0.30503416320185783, 0.4341849137243666, 0.19714312154012825, 0.5904045080679433, 0.10891825045634008, 0.6949605704841074, 0.7523635855803892, 0.8405431561416944, 0.6596760569453475, 0.7802018466088256, 0.9406459881826373, 0.028458619177697642, 0.686194890645648, 0.6375815092596757, 0.5814599189289762, 0.45463224003940067, 0.021684946854834988, 0.055838928272145934, 0.18312527356430858, 0.9459317157581488, 0.43405229091293374, 0.01674068278331136, 0.8006420540454028, 0.383751096124356, 0.41009634184300503, 0.6067312588311528, 0.7272538758593727, 0.824149880978979, 0.10557138522553336, 0.9467111748788416, 0.20746276983131878, 0.8743860194002515, 0.7495455265120637, 0.46948964639871776, 0.3718915744286971, 0.4668543681705316, 0.5595220981253388, 0.34623179191310105, 0.8666460962189857, 0.18795554129907963, 0.024404536039133104, 0.44536025070281554, 0.8785465434194476, 0.8070133240928774, 0.7735398534749577, 0.5594128768558738, 0.3105811936433562, 0.623377894291806, 0.42393664964194955, 0.3007869891367447, 0.9165296668546321, 0.4300385246349278, 0.562026753058214, 0.31565167986756504, 0.27711736490755, 0.49998261793605214, 0.6185522479814858, 0.7742784623500055, 0.2331167645358907, 0.6811825768087665, 0.8474253822370598, 0.051441973793744725, 0.8742029103860041, 0.33696720008516184, 0.2220788661377981, 0.6431555099758184, 0.746244356573601, 0.9020074055661763, 0.5027600716952823, 0.1841524660881267, 0.41228739113433405, 0.564634713253446, 0.5203486550963307, 0.6197032692096887, 0.615513815818285, 0.4255646218064673, 0.7480245632325067, 0.4374964504869727, 0.8113181909548437, 0.7002788178128234, 0.930057907150934, 0.08921892743397504, 0.07842573633516994, 0.8313988437953759, 0.30024379568278836, 0.11542043641726706, 0.028871447702799945, 0.6440198681817485, 0.7435178253112837, 0.40565573685287054, 0.05457974205324944, 0.6951643520830729, 0.7699273377320647, 0.2934123488766337, 0.29079763244855894, 0.7698200240792725, 0.7107824078318179, 0.39177309792431, 0.8709642493379329, 0.8036192832765259, 0.7496356221899658, 0.2976310146283583, 0.9062287990345668, 0.0385471452014495, 0.04326264765980925, 0.021495159778825833, 0.35698178332320446, 0.9616607228032605, 0.8758570814518778, 0.6211941794113294, 0.46053707483688544, 0.17749216495621617, 0.8299062901566412, 0.9042960582329184, 0.8035220763607114, 0.35490130334240605, 0.10218991345249802, 0.5144624069972654, 0.6077042300554744, 0.06678892593555155, 0.3051641737825565, 0.35514734258038655, 0.916226196505605, 0.8639003857582148, 0.7187357812077981, 0.3994968464482591, 0.5956003940258635, 0.5360221754338779, 0.12969545638897262, 0.9567768199533195, 0.3462570910029834, 0.4371931249505044, 0.548573597586304, 0.7227074312761246, 0.671923814164644, 0.6532717201570521, 0.5131257808251704, 0.16236597694000637, 0.6141483216470036, 0.49065146459090814, 0.5057377902654527, 0.8718970406906583, 0.7541874999524453, 0.17932974413298572, 0.8625348891105667, 0.5473987305464777, 0.9175353077980257, 0.4180608690840789, 0.6224492368656882, 0.49961693965375314, 0.28301678191348745, 0.6092265845186635, 0.7021957139297794, 0.08000372614534122, 0.6127223506871815, 0.12877001247542846, 0.5056206215702727, 0.7698434820649745, 0.03729947108220399, 0.42605762507089473, 0.11329038071943909, 0.5332390335956054, 0.5658172563765423, 0.9134834883207194, 0.39957850170379905, 0.2092775742398068, 0.7037982864609181, 0.7563796124561704, 0.9880444460862121, 0.5172603745678069, 0.504391112364453, 0.7036178372396958, 0.35059489764088153, 0.8489803506366923, 0.48365389427033806, 0.9506640287107418, 0.715950037768444, 0.8834527293458425, 0.04325105977673146, 0.3128681549411839, 0.4413597135033203, 0.5641462358315511, 0.8608468937477413, 0.14604440255285472, 0.5061510561370258, 0.1922332493669038, 0.8497416390817859, 0.829092314124154, 0.19842298057591512, 0.7992720203875622, 0.9307543031411136, 0.31172697399048693, 0.7475970491026749, 0.11107526980019038, 0.8565829265834732, 0.009904253061437952, 0.05840151234278229, 0.5641682936375996, 0.09233108349916634, 0.9990777847469534, 0.9261954056326351, 0.5443892584069489, 0.005257919880084335, 0.57556635204602, 0.2505073813800971, 0.4887839901703743, 0.0910778853221713, 0.2777080084768017, 0.6465752039843234, 0.1402500815542422, 0.7323870636008557, 0.6886082288020439, 0.0971325438921782, 0.873593637155617, 0.5957325396803912, 0.47698048291245515, 0.5697965719753848, 0.5049936385514854, 0.7167869683056091, 0.5330419240600595, 0.8396229015436258, 0.1793739588601884, 0.8126372345483, 0.34829637402091546, 0.08468083499215051, 0.06291611905410097, 0.40047096726994785, 0.9170578073263495, 0.4348096892240936, 0.059058673850685216, 0.42069283783520506, 0.04759243876611651, 0.3623291308244055, 0.5506257727313975, 0.2742299813100154, 0.11773318579965336, 0.5673194197617327, 0.355165539075596, 0.23565923009819428, 0.10177490837978377, 0.8292161250657494, 0.7045280172789363, 0.7092144139789265, 0.4872026019879656, 0.538097321271237, 0.7072270572043323, 0.06903829248488051, 0.45906010673760633, 0.4781217457823192, 0.16494119486870262, 0.9481693464076955, 0.9761312347224527, 0.8851844184481438, 0.2709935552597104, 0.49535627804570864, 0.5764703269353461, 0.44277920787438396, 0.23310882757494603, 0.8397298604569601, 0.6443087227636258, 0.8102317029833325, 0.6380643110052716, 0.4352434443258309, 0.4373723606067367, 0.03407197020943986, 0.051327440995096514, 0.8476742936795423, 0.9270971128240528, 0.4988517662841063, 0.23544714066192124, 0.0680757458473022, 0.006478821672326959, 0.05036359098131076, 0.633006704484185, 0.4823462528984263, 0.31241488518946237, 0.6027427831384861, 0.5968576533639238, 0.9525674002397705, 0.1808262241597047, 0.6628706313955393, 0.33707623001765596, 0.7251843394821078, 0.9912790021833987, 0.4916347506343979, 0.9264263005397082, 0.9526542460250946, 0.3716490860079617, 0.538670456790784, 0.2680150580710617, 0.05156222840970183, 0.7441725543869716, 0.7742572731078238, 0.6664722118771624, 0.7416816487920486, 0.7052399014247716, 0.005588593465904534, 0.43285634619410546, 0.3472509147415241, 0.21862849485688982, 0.40841151609598847, 0.7613456295344699, 0.2916417389069094, 0.281682705568727, 0.26596475091496896, 0.16737888003952053, 0.6169638921658713, 0.3550306146974478, 0.9071068390567893, 0.22336582613090666, 0.2856246860224741, 0.8106487891848176, 0.40910418725766334, 0.21600220945513068, 0.6200689656104054, 0.997385492655497, 0.21324772658593857, 0.1792235146040162, 0.571572447344655, 0.47294193386705813, 0.3389465770394191, 0.39954488108949704, 0.7887634507391187, 0.19222138023125043, 0.7146500241822473, 0.7187030596623396, 0.8746377272456051, 0.8398593325448436, 0.7014497147050506, 0.4821247455633702, 0.020514480590571253, 0.5118029931590595, 0.13809464267438848, 0.6798813660937794, 0.9688116437839488, 0.8090385700462209, 0.8617296499318542, 0.0610888196081699, 0.9297603926326202, 0.2069631831752189, 0.49062517753710194, 0.6551534278214852, 0.4373582885931502, 0.025105547146966604, 0.947765233555461, 0.6624644788143533, 0.6081120134960533, 0.06676322978333268, 0.4855521307549493, 0.5416568553910562, 0.8038581421961926, 0.5059582453436794, 0.7829858691678123, 0.2391504666161637, 0.921282585516366, 0.9266466016017137, 0.9650573553087818, 0.9030133978549378, 0.9138439231342235, 0.7260748859017618, 0.48931931054816596, 0.9477357958951085, 0.7470370248423088, 0.8646438442393617, 0.7499609020337648, 0.11603418249330764, 0.015083720268691614, 0.7752991608976825, 0.32521552456033337, 0.7421446580269376, 0.3657723233623237, 0.5141987290205983, 0.9562236721683256, 0.45194155402262665, 0.680921157471833, 0.9111025511883462, 0.4318915323967331, 0.925535061464924, 0.6191637551634921, 0.9041538978958329, 0.636475627099519, 0.20402665887697913, 0.671307685248755, 0.9400855922962167, 0.2330312829960237, 0.6438247379456898, 0.26082249832468823, 0.3188784666161744, 0.09713817492960711, 0.3915916045923539, 0.15162822199958556, 0.060257932474917966, 0.688434842865665, 0.31547573943505985, 0.06230541201403825, 0.7730857472942804, 0.10058067603701719, 0.1311552530603735, 0.14209412677358946, 0.19233695226150563, 0.6186330969876527, 0.8360898339490553, 0.54153836172022, 0.15533977993600204, 0.055926662442861735, 0.9354675188150726, 0.5224643503065718, 0.07432291249313354, 0.8679244099816513, 0.8581417967938532, 0.5753473065890775, 0.1485586610981194, 0.45863032324434594, 0.0823143296507978, 0.737299344756597, 0.20900543217013395, 0.019349179587765858, 0.9287023393871775, 0.30559476978368116, 0.4276196122306571, 0.938907210445459, 0.5195819721409203, 0.8517761911490239, 0.36245953708986306, 0.032843136283026035, 0.14503492378751257, 0.33383449761700085, 0.6976665312992817, 0.6399510661975897, 0.5372713907223594, 0.2365548791866996, 0.30891742189846505, 0.8206934489041221, 0.7799012072855613, 0.8760136539976576, 0.6530605780104675, 0.45124431762074807, 0.8496559257406205, 0.05577460535340131, 0.4236378388372895, 0.196338320060314, 0.6425691558199718, 0.25882940141083227, 0.9093140010481955, 0.083622402237035, 0.6763438122383616, 0.38793273540011824, 0.09396589297984644, 0.24664801588472743, 0.7273501052780645, 0.16481104419164921, 0.07856464509910391, 0.5220339770914201, 0.23655302817057874, 0.11595937827868501, 0.05436560362759264, 0.6225309275234828, 0.7892690138059433, 0.5352486452914279, 0.6330100303710987, 0.06284051261238073, 0.896167086237767, 0.24480328907039994, 0.9541669336884637, 0.47517431158536, 0.14748428404231234, 0.24975395777254084, 0.9743366026272987, 0.8804963656213821, 0.606257858507387, 0.9049015682915633, 0.6325285846858149, 0.27576473756451475, 0.8376209514575742, 0.4736773097242755, 0.8007165698980012, 0.3683313895867722, 0.4714928376535322, 0.7801490991167184, 0.6704153289248763, 0.7758264354215532, 0.9712984319365751, 0.08225110355599838, 0.6105175068565182, 0.7018874327201039, 0.5015419814926569, 0.506597125206996, 0.8058854817022504, 0.8830863985797442, 0.9293900035286071, 0.8074960424049121, 0.9698203572525096, 0.8766893742900868, 0.7162787249486939, 0.9492509202396994, 0.45996988335981, 0.4148409053861623]

